from __future__ import print_function
import USB3014Device
import time
import threading
# import multiprocessing
import os


def findDevByID(dev_id, find_times, find_delay):
    for i in range(0, find_times):
        for device in dev3014Iter.iterDevice():
            if hash(device) in dev_hash_list:
                continue
            device.open()
            if dev_id == device.id():
                addDevHash(dev_id, hash(device))
                return device
        else:
            time.sleep(find_delay / 1000.0)
    else:
        return None


def findAutoDevByID(auto_device, find_times, find_delay):
    # type: (USB3014Device.Auto3014Device, int ,int) -> bool
    for i in range(0, find_times):
        if auto_device.refresh():
            return True
        else:
            time.sleep(find_delay / 1000.0)
    else:
        return False


def switchSpeed(device, dev_id, spd):
    device.switchSpeedMode(spd)
    return findDevByID(dev_id, 10, 1000)


def switchSpeedAutoDevice(auto_device, spd):
    # type: (USB3014Device.Auto3014Device, USB3014Device.SpeedMode) -> bool
    auto_device.device.switchSpeedMode(spd)
    if findAutoDevByID(auto_device, 10, 1000):
        return auto_device.device.speedMode() == spd
    else:
        return False


def addDevHash(dev_id, device_hash):
    with hash_list_locker:
        dev_hash_list[device_hash] = dev_id


def removeDevHash(device_hash):
    with hash_list_locker:
        dev_hash_list.pop(device_hash)


def cleanDevHash():
    with hash_list_locker:
        dev_hash_list[:] = {}


def testItem(device):
    dev_id = device.id()
    print("id = ", dev_id, " test device id = ", dev_id, "fw = ", device.firmware(), "bcdUSB = 0x%x" % device.bcdUSB())
    print("id = ", dev_id, "int test res = ", device.intTest())
    print("id = ", dev_id, "bulk test res = ", device.bulkTest())
    print("id = ", dev_id, "read speed = ", device.readSpeedTest(256, 512, 10))
    print("id = ", dev_id, "write speed = ", device.writeSpeedTest(512, 512, 5))


def testByID(dev_id):
    device = findDevByID(dev_id, 10, 1000)
    if device is None:
        print("device lost before start test id = %d" % dev_id)
        return False
    device_hash = hash(device)
    addDevHash(dev_id, device_hash)
    testItem(device)
    print("id = ", dev_id, "switching to usb3.0")
    device = switchSpeed(device, dev_id, USB3014Device.SpeedMode.SS)
    if device is None:
        print("device lost when switching to ss id = %d" % dev_id)
        return False
    addDevHash(dev_id, device_hash)
    testItem(device)
    device.reset(True)


def testByAutoDevice(auto_device):
    # type: (USB3014Device.Auto3014Device) -> None
    if not auto_device.open():
        print("fail to open device, %s" % auto_device)
        return
    else:
        print("start test %s" % auto_device)
    dev_id = auto_device.id
    testItem(auto_device.device)
    print("id = ", dev_id, "switching to usb3.0")
    switchSpeedAutoDevice(auto_device, USB3014Device.SpeedMode.SS)
    if auto_device.lost():
        print("device lost when switching to ss, %s" % auto_device)
        return
    testItem(auto_device.device)
    auto_device.device.reset(True)


if __name__ == "__main__":
    dev3014Iter = USB3014Device.USB3014DeviceIter()
    id_list = [15, 1, 14]
    dev_hash_list = {}
    hash_list_locker = threading.Lock()
    thread_list = []

    is_using_auto_device = True
    if is_using_auto_device:
        for dev_id in id_list:
            auto3014 = USB3014Device.Auto3014Device(dev3014Iter, dev_id)
            thd = threading.Thread(target=testByAutoDevice, args=(auto3014,))
            thd.daemon = True
            thd.start()
            thread_list.append(thd)
    else:
        for dev_id in id_list:
            thd = threading.Thread(target=testByID, args=(dev_id,))
            thd.daemon = True
            thd.start()
            thread_list.append(thd)

    for thd in thread_list:
        thd.join()

    print("test finished")
